#Singleton class for Wakari Data S3 Loader
class _WikiLogReader(object):

    _instance = None

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = super(_WikiLogReader, cls).__new__(
                                cls, *args, **kwargs)
        return cls._instance

    def __init__(self):
        self._dataAdapter = None
        
    def make_adapter(self,key):
        import iopro 

        #change location and file to S3 when created
        loc = 'WikiData/'
        wikiFile = 'pagecounts-20110101-130000.gz'

        self._dataAdapter = iopro.text_adapter(loc+wikiFile, delimiter=' ', compression='gzip', field_names=False)

        self._dataAdapter.set_field_types({0:'object', 1:'object', 2:'i4', 3:'i8'})

        # Set dtype for each field in record
        self._dataAdapter.set_converter(1, str)

        # build index of records and save index to file
        self.indexArray, self.gzipIndexArray = self._dataAdapter.create_index()

        self._dataAdapter.set_index((self.indexArray, self.gzipIndexArray))

    def __getitem__(self,key):

        #lazy evaluation of adapter.  Need to properly handle events where data cannot be loaded
        if not self._dataAdapter:
            self.make_adapter(key)

        return self._dataAdapter[key]
        


wikilog = _WikiLogReader()

#example usage:
   # from wakaridata import wikilog
   # wikilog[10:232] 